% BT revision 1 - 2016-10-14

clear
clc

DATA = load('baltic_raw_data_01.mat');

fields = fieldnames(DATA);
idx = isnan(DATA.INC) | isnan(DATA.GENdist); 
for i = 1:numel(fields)
    tmp = DATA.(fields{i});
    DATA.(fields{i}) = tmp(idx==0,:);
end
clear fields idx tmp i

DATA.hWTPl(DATA.WTPY==0) = -Inf;
DATA.hWTPh(DATA.WTPY==0) = 0;
DATA.fWTPl(DATA.WTPY==0) = -Inf;
DATA.fWTPh(DATA.WTPY==0) = 0;

DATA.bounds_h = [DATA.hWTPl./DATA.PPP DATA.hWTPh./DATA.PPP];
DATA.bounds_f = [DATA.fWTPl./DATA.PPP DATA.fWTPh./DATA.PPP];

% GENdist - distance to the sea in 100 km
% INC - household income in 1000 EUR @ PPP
DATA.GENdist(DATA.GENdist < 0.01) = 0.01;

for ctr = 1:9
    INCm(ctr) = nanmean(DATA.INC(DATA.CTR == ctr));
    GENdistm(ctr) = nanmean(DATA.GENdist(DATA.CTR == ctr));
end


DATA.X = zeros(size(DATA.INC,1),0);
INPUT.NamesX = {'None'};

% INPUT.Spike = 1; % allow for jump density at 0

INPUT.HessEstFix = 3; % available options: 0 - retained from optimization, 1 - BHHH based, 2 - numerical high-precision Jacobian based, 3 - numerical Hessian based
% INPUT.SimStats = 1; % simulate descriptive statistics of the distribution

Countries = {'DK','EE','FI','DE','LV','LIT','PL','RU','SE'};

Results_h = [];
Results_f = [];

% baseline:
B0 = [-2120.73696678672,-1122.74643746932,-4008.19416605819,-2765.11205320879,-1377.53206947545,-1240.89722480110,-3956.69988130247,-2332.44097352750,-2752.57792753734;-2146.30832085073,-1123.09989632315,-4047.85218876068,-2852.12474590115,-1403.23270614527,-1257.44792921261,-3888.16207249586,-2229.93524587531,-2764.44886561262;3.29054058217251,-0.143082688460184,10.9149366058983,2.65353384555487,-1.33540147106937,-1.51524226054411,0.712451233712440,-29.4362564311868,36.8542167082301;83.3112350274034,59.6895088336394,77.0664309282337,48.8924048623135,14.3850488007303,22.9380044130189,29.8613805946770,50.8159501602020,151.094251853993;2.70327838632304,1.31929328569862,14.0129486491712,2.59658310429275,-1.83382755146057,-0.549955223334505,1.51457519673734,-30.9510822136809,53.1886765179038;90.1125603226729,76.5300101358992,101.866655930222,72.8430087162996,17.9746899870712,28.9378673062291,36.2774429570359,55.7306810076452,169.886552782192];


INPUT.Algorithm = 'sqp';% 'search' 'interior-point' 'sqp' 'active-set' 'trust-region-reflective'

LLdif = -1;
while min(LLdif(:)) < -1e-6

LL0 = reshape(B0(1:2,:)',[1,18]); 
B0_h = B0(3:(size(B0,1)-2)./2+2,:);
B0_f = B0((size(B0,1)-2)./2+3:end,:);

Results_h = [];
Results_f = [];
for ctr = 1:9
    INPUT.bounds = DATA.bounds_h(DATA.CTR == ctr,:);
    INPUT.X = DATA.X(DATA.CTR == ctr,:);
    Results_tmp = DistFit(INPUT,0,B0_h(:,ctr));
    Results_tmp.CTR = Countries{ctr};
    Results_h = [Results_h,Results_tmp];
    
    INPUT.bounds = DATA.bounds_f(DATA.CTR == ctr,:);
    Results_tmp = DistFit(INPUT,0,B0_f(:,ctr));
    Results_tmp.CTR = Countries{ctr};
    Results_f = [Results_f,Results_tmp];
end
clear Results_tmp
% Results_h = orderfields(Results_h,[17,1:16]);
% Results_f = orderfields(Results_f,[17,1:16]);
Results_h = orderfields(Results_h,[numel(fields(Results_h)),1:numel(fields(Results_h))-1]);
Results_f = orderfields(Results_f,[numel(fields(Results_f)),1:numel(fields(Results_f))-1]);


% B0_h = B0_h2;
% B0_f = B0_f2;

LL1 = [Results_h.fval,Results_f.fval];
LLdif = reshape(LL0 - LL1,[9,2])'
B0_h_new = [Results_h.beta];
B0_h(:,LLdif(1,:)<0) = B0_h_new(:,LLdif(1,:)<0);
B0_f_new = [Results_f.beta];
B0_f(:,LLdif(2,:)<0) = B0_f_new(:,LLdif(2,:)<0);
% LL0(LL1>LL0) = LL1(LL1>LL0);

% LL0_new = [Results_h.fval,Results_f.fval];
% idx = reshape(LL0 < LL0_new,[2,9]);
% B0_h_new = [Results_h.beta];
% B0_f_new = [Results_f.beta];
% B0_h(:,idx(1,:)) = B0_h_new(:,idx(1,:))
% B0_f(:,idx(2,:)) = B0_f_new(:,idx(2,:))
% 
% B0_h = B0_h_new;
% B0_f = B0_f_new;
B0 = [reshape(LL1,[9,2])'; B0_h; B0_f];

% save FT_distance

end

LL1 = reshape(LL1, [9,2])



%% simulate moments of spike distribution


samp1 = 1e5;
samp2 = 1e5; % 1e5 x 1e5 = 20 min
WTP_stats = zeros(samp2,4);
for ctr = 1:9
    % half:
    betas_ctr = mvnrnd(Results_h(ctr).beta,Results_h(ctr).ihess,samp1);
    mu_tmp = betas_ctr(:,1);
    sigma_tmp = betas_ctr(:,2);
    parfor i = 1:samp2
        WTP_i = normrnd(mu_tmp(i),sigma_tmp(i),[samp2,1]);
        WTP_i(WTP_i < 0) = 0;
        WTP_stats(i,:) = [mean(WTP_i), median(WTP_i), std(WTP_i), normcdf(0,mu_tmp(i),sigma_tmp(i))];
    end
    Results_h(ctr).WTP_stats = [median(WTP_stats);std(WTP_stats)];
    % full:
    betas_ctr = mvnrnd(Results_f(ctr).beta,Results_f(ctr).ihess,samp1);
    mu_tmp = betas_ctr(:,1);
    sigma_tmp = betas_ctr(:,2);
    parfor i = 1:samp2
        WTP_i = normrnd(mu_tmp(i),sigma_tmp(i),[samp2,1]);
        WTP_i(WTP_i < 0) = 0;
        WTP_stats(i,:) = [mean(WTP_i), median(WTP_i), std(WTP_i), normcdf(0,mu_tmp(i),sigma_tmp(i))];
    end
    Results_f(ctr).WTP_stats = [median(WTP_stats);std(WTP_stats)];
end; 
clear betas_ctr mu_tmp sigma_tmp WTP_i WTP_stats samp1 samp2 ctr i

WTP_stats_h = reshape([Results_h.WTP_stats],[2,4,9]);
WTP_stats_f = reshape([Results_f.WTP_stats],[2,4,9]);


%% BT - one-to-one, PPP


means_h = squeeze(WTP_stats_h(1,1,:));
means_f = squeeze(WTP_stats_f(1,1,:));
se_h = squeeze(WTP_stats_h(2,1,:));
se_f = squeeze(WTP_stats_f(2,1,:));

% 1e5 x 1e5:
means_h = [34.9324271472054;23.7504653056327;36.5064539036483;20.8510809180655;5.10209729585991;8.41861368350906;12.2779228774996;8.86682023165685;80.4687822915646];
means_f = [37.3351217628395;31.1826306219991;48.0554208664498;30.3857760099688;6.29790265480797;11.2741945077405;15.2456054742287;10.0946883416700;97.7235194450257];
se_h = [1.70587932827281;1.69246454298268;1.27926141975129;0.897218578415484;0.326215309452767;0.558507394163843;0.474457378490646;0.555126350835681;3.88835082309821];
se_f = [1.83800931529212;2.20997159463781;1.62169706659524;1.30339341761000;0.404362186834984;0.752880002664411;0.580229118990209;0.623300126133862;3.93083187651872];

el = 0;
TEh1 = TEMatrix(means_h);
TEf1 = TEMatrix(means_f);
MTLh1 = MTLMatrix(means_h,se_h);
MTLf1 = MTLMatrix(means_f,se_f);

% BT - one-to-one, el = 0.2824
el = 0.2824;
TEh2 = TEMatrix(means_h,INCm,el);
TEf2 = TEMatrix(means_f,INCm,el);
MTLh2 = MTLMatrix(means_h,se_h,means_h',se_h',INCm,INCm,el);
MTLf2 = MTLMatrix(means_f,se_f,means_f',se_f',INCm,INCm,el);

% BT - one-to-one, el = 1
el = 1;
TEh3 = TEMatrix(means_h,INCm,el);
TEf3 = TEMatrix(means_f,INCm,el);
MTLh3 = MTLMatrix(means_h,se_h,means_h',se_h',INCm,INCm,el);
MTLf3 = MTLMatrix(means_f,se_f,means_f',se_f',INCm,INCm,el);

TEf1(diag(ones(size(TEf1,1),1))==1) = NaN;
TEh1(diag(ones(size(TEh1,1),1))==1) = NaN;
TEf2(diag(ones(size(TEf2,1),1))==1) = NaN;
TEh2(diag(ones(size(TEh2,1),1))==1) = NaN;
TEf3(diag(ones(size(TEf3,1),1))==1) = NaN;
TEh3(diag(ones(size(TEh3,1),1))==1) = NaN;

MTLf1(diag(ones(size(MTLf1,1),1))==1) = NaN;
MTLh1(diag(ones(size(MTLh1,1),1))==1) = NaN;
MTLf2(diag(ones(size(MTLf2,1),1))==1) = NaN;
MTLh2(diag(ones(size(MTLh2,1),1))==1) = NaN;
MTLf3(diag(ones(size(MTLf3,1),1))==1) = NaN;
MTLh3(diag(ones(size(MTLh3,1),1))==1) = NaN;

AvgTE = [...
nanmean(abs(reshape([TEf1,TEh1],[2*9^2,1]))),...
nanmean(abs(reshape([TEf2,TEh2],[2*9^2,1]))),...
nanmean(abs(reshape([TEf3,TEh3],[2*9^2,1])));

nanmedian(abs(reshape([TEf1,TEh1],[2*9^2,1]))),...
nanmedian(abs(reshape([TEf2,TEh2],[2*9^2,1]))),...
nanmedian(abs(reshape([TEf3,TEh3],[2*9^2,1])))];

AvgMTL = [...
nanmean(abs(reshape([MTLf1,MTLh1],[2*9^2,1]))),...
nanmean(abs(reshape([MTLf2,MTLh2],[2*9^2,1]))),...
nanmean(abs(reshape([MTLf3,MTLh3],[2*9^2,1])));

nanmedian(abs(reshape([MTLf1,MTLh1],[2*9^2,1]))),...
nanmedian(abs(reshape([MTLf2,MTLh2],[2*9^2,1]))),...
nanmedian(abs(reshape([MTLf3,MTLh3],[2*9^2,1])))];

out1 = [LL1, zeros(9,1), TEf1, zeros(9,1), TEh1, zeros(9,1), MTLf1, zeros(9,1), MTLh1, zeros(9,1), [AvgTE(:,1);zeros(7,1)], zeros(9,1), [AvgMTL(:,1);zeros(7,1)]];
out2 = [LL1, zeros(9,1), TEf2, zeros(9,1), TEh2, zeros(9,1), MTLf2, zeros(9,1), MTLh2, zeros(9,1), [AvgTE(:,2);zeros(7,1)], zeros(9,1), [AvgMTL(:,2);zeros(7,1)]];
out3 = [LL1, zeros(9,1), TEf3, zeros(9,1), TEh3, zeros(9,1), MTLf3, zeros(9,1), MTLh3, zeros(9,1), [AvgTE(:,3);zeros(7,1)], zeros(9,1), [AvgMTL(:,3);zeros(7,1)]];




%% what el minimizes TE?


OptimOpt = optimoptions('fminunc');
OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.GradObj = 'off';
OptimOpt.Hessian = 'off'; %'user-supplied';
OptimOpt.FinDiffType = 'central'; % ('forward')
OptimOpt.MaxIter = 1e4; %1e4; %1e4; % no of iterations
OptimOpt.TolFun = 1e-9;
OptimOpt.TolX = 1e-9;
OptimOpt.FunValCheck = 'on';
OptimOpt.Diagnostics = 'on';
OptimOpt.MaxFunEvals = 1e4;
OptimOpt.OutputFcn = @outputf;

x0 = 1;

TEfun = @(B) nanmean(reshape(abs([TEMatrix(means_h,INCm,B),TEMatrix(means_f,INCm,B)]),[2*9^2,1]));
[best_el_mean,min_meanTE] = fminunc(TEfun,x0,OptimOpt)

TEfun = @(B) nanmedian(reshape(abs([TEMatrix(means_h,INCm,B),TEMatrix(means_f,INCm,B)]),[2*9^2,1]));
[best_el_median,min_medianTE] = fminunc(TEfun,x0,OptimOpt)

clear x0 TEfun OptimOpt el


%% 2. Use GDP instead of the observed mean income levels


GDPm = [37741.929, 20182.126, 36723.292, 37935.519, 15448.091, 18769.507, 20136.903, 16687.376, 40613.817];
% PPP = [8.802, 0.592, 0.965, 0.831, 0.394, 1.717, 1.952, 22.527, 9.368];

% means_h = squeeze(WTP_stats_h(1,1,:));
% means_f = squeeze(WTP_stats_f(1,1,:));
% se_h = squeeze(WTP_stats_h(2,1,:));
% se_f = squeeze(WTP_stats_f(2,1,:));

el = 0;
TEh4 = TEMatrix(means_h);
TEf4 = TEMatrix(means_f);
MTLh4 = MTLMatrix(means_h,se_h);
MTLf4 = MTLMatrix(means_f,se_f);

% BT - one-to-one, el = 0.2824
el = 0.2824;
TEh5 = TEMatrix(means_h,GDPm,el);
TEf5 = TEMatrix(means_f,GDPm,el);
MTLh5 = MTLMatrix(means_h,se_h,means_h',se_h',GDPm,GDPm,el);
MTLf5 = MTLMatrix(means_f,se_f,means_f',se_f',GDPm,GDPm,el);

% BT - one-to-one, el = 1
el = 1;
TEh6 = TEMatrix(means_h,GDPm,el);
TEf6 = TEMatrix(means_f,GDPm,el);
MTLh6 = MTLMatrix(means_h,se_h,means_h',se_h',GDPm,GDPm,el);
MTLf6 = MTLMatrix(means_f,se_f,means_f',se_f',GDPm,GDPm,el);

TEf4(diag(ones(size(TEf4,1),1))==1) = NaN;
TEh4(diag(ones(size(TEh4,1),1))==1) = NaN;
TEf5(diag(ones(size(TEf5,1),1))==1) = NaN;
TEh5(diag(ones(size(TEh5,1),1))==1) = NaN;
TEf6(diag(ones(size(TEf6,1),1))==1) = NaN;
TEh6(diag(ones(size(TEh6,1),1))==1) = NaN;

MTLf4(diag(ones(size(MTLf4,1),1))==1) = NaN;
MTLh4(diag(ones(size(MTLh4,1),1))==1) = NaN;
MTLf5(diag(ones(size(MTLf5,1),1))==1) = NaN;
MTLh5(diag(ones(size(MTLh5,1),1))==1) = NaN;
MTLf6(diag(ones(size(MTLf6,1),1))==1) = NaN;
MTLh6(diag(ones(size(MTLh6,1),1))==1) = NaN;

AvgTE2 = [...
nanmean(abs(reshape([TEf4,TEh4],[2*9^2,1]))),...
nanmean(abs(reshape([TEf5,TEh5],[2*9^2,1]))),...
nanmean(abs(reshape([TEf6,TEh6],[2*9^2,1])));

nanmedian(abs(reshape([TEf4,TEh4],[2*9^2,1]))),...
nanmedian(abs(reshape([TEf5,TEh5],[2*9^2,1]))),...
nanmedian(abs(reshape([TEf6,TEh6],[2*9^2,1])))];

AvgMTL2 = [...
nanmean(abs(reshape([MTLf4,MTLh4],[2*9^2,1]))),...
nanmean(abs(reshape([MTLf5,MTLh5],[2*9^2,1]))),...
nanmean(abs(reshape([MTLf6,MTLh6],[2*9^2,1])));

nanmedian(abs(reshape([MTLf4,MTLh4],[2*9^2,1]))),...
nanmedian(abs(reshape([MTLf5,MTLh5],[2*9^2,1]))),...
nanmedian(abs(reshape([MTLf6,MTLh6],[2*9^2,1])))];

out4 = [LL1, zeros(9,1), TEf4, zeros(9,1), TEh4, zeros(9,1), MTLf4, zeros(9,1), MTLh4, zeros(9,1), [AvgTE2(:,1);zeros(7,1)], zeros(9,1), [AvgMTL2(:,1);zeros(7,1)]];
out5 = [LL1, zeros(9,1), TEf5, zeros(9,1), TEh5, zeros(9,1), MTLf5, zeros(9,1), MTLh5, zeros(9,1), [AvgTE2(:,2);zeros(7,1)], zeros(9,1), [AvgMTL2(:,2);zeros(7,1)]];
out6 = [LL1, zeros(9,1), TEf6, zeros(9,1), TEh6, zeros(9,1), MTLf6, zeros(9,1), MTLh6, zeros(9,1), [AvgTE2(:,3);zeros(7,1)], zeros(9,1), [AvgMTL2(:,3);zeros(7,1)]];




%% what el minimizes TE?


OptimOpt = optimoptions('fminunc');
OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.GradObj = 'off';
OptimOpt.Hessian = 'off'; %'user-supplied';
OptimOpt.FinDiffType = 'central'; % ('forward')
OptimOpt.MaxIter = 1e4; %1e4; %1e4; % no of iterations
OptimOpt.TolFun = 1e-9;
OptimOpt.TolX = 1e-9;
OptimOpt.FunValCheck = 'on';
OptimOpt.Diagnostics = 'on';
OptimOpt.MaxFunEvals = 1e4;
OptimOpt.OutputFcn = @outputf;

x0 = 1;

TEfun = @(B) nanmean(reshape(abs([TEMatrix(means_h,GDPm,B),TEMatrix(means_f,GDPm,B)]),[2*9^2,1]));
[best_el_mean2,min_meanTE2] = fminunc(TEfun,x0,OptimOpt)

TEfun = @(B) nanmedian(reshape(abs([TEMatrix(means_h,GDPm,B),TEMatrix(means_f,GDPm,B)]),[2*9^2,1]));
[best_el_median2,min_medianTE2] = fminunc(TEfun,x0,OptimOpt)

clear x0 TEfun OptimOpt el
