% BT revision 1 - 2016-10-14

clear
clc

DATA = load('baltic_raw_data_01.mat');

% drop missing inc or GENdist:
fields = fieldnames(DATA);
idx = isnan(DATA.INC) | isnan(DATA.GENdist);
for i = 1:numel(fields)
    tmp = DATA.(fields{i});
    DATA.(fields{i}) = tmp(idx==0,:);
end
clear fields idx tmp i

DATA.hWTPl(DATA.WTPY==0) = -Inf;
DATA.hWTPh(DATA.WTPY==0) = 0;
DATA.fWTPl(DATA.WTPY==0) = -Inf;
DATA.fWTPh(DATA.WTPY==0) = 0;

DATA.bounds_h = [DATA.hWTPl./DATA.PPP DATA.hWTPh./DATA.PPP];
DATA.bounds_f = [DATA.fWTPl./DATA.PPP DATA.fWTPh./DATA.PPP];

% GENdist - distance to the sea in 100 km
% INC - household income in 1000 EUR @ PPP

DATA.GENdist(DATA.GENdist < 0.01) = 0.01;
% for ctr = 1:9
%     DATA.GENdist(DATA.CTR == ctr) = min(DATA.GENdist(DATA.CTR == ctr),quantile(DATA.GENdist(DATA.CTR == ctr),0.975));
% end
weights = (1./sum(dummyvar(DATA.CTR)))/9*size(DATA.INC,1);

DATA.W = ones(size(DATA.CTR));
for ctr = 1:9
    INCm(ctr) = nanmean(DATA.INC(DATA.CTR == ctr));
    GENdistm(ctr) = nanmean(DATA.GENdist(DATA.CTR == ctr));
    logINCm(ctr) = nanmean(log(DATA.INC(DATA.CTR == ctr)));
    logGENdistm(ctr) = nanmean(log(DATA.GENdist(DATA.CTR == ctr)));
    expINCm(ctr) = nanmean(exp(DATA.INC(DATA.CTR == ctr)));
    expGENdistm(ctr) = nanmean(exp(DATA.GENdist(DATA.CTR == ctr)));

    INCm12(ctr,1:2) = nanmean([DATA.INC(DATA.CTR == ctr),DATA.INC(DATA.CTR == ctr).^2]);
    GENdistm12(ctr,1:2) = nanmean([DATA.GENdist(DATA.CTR == ctr),DATA.GENdist(DATA.CTR == ctr).^2]);
    
    INCm123(ctr,1:3) = nanmean([DATA.INC(DATA.CTR == ctr),DATA.INC(DATA.CTR == ctr).^2,DATA.INC(DATA.CTR == ctr).^3]);
    GENdistm123(ctr,1:3) = nanmean([DATA.GENdist(DATA.CTR == ctr),DATA.GENdist(DATA.CTR == ctr).^2,DATA.GENdist(DATA.CTR == ctr).^3]);
    
    INCo(ctr) = nanmean(DATA.INC(DATA.CTR ~= ctr));
    GENdisto(ctr) = nanmean(DATA.GENdist(DATA.CTR ~= ctr));
    DATA.W(DATA.CTR==ctr) = weights(ctr);
end

% DATA.X = zeros(size(DATA.INC,1),0);
% INPUT.NamesX = {'None'};

% DATA.X = [DATA.INC];
% INPUT.NamesX = {'Income'};

% DATA.X = [DATA.INC DATA.INC.^2];
% INPUT.NamesX = {'Income','Income^2'};

% DATA.X = [DATA.INC DATA.INC.^2 DATA.INC.^3];
% INPUT.NamesX = {'Income','Income^2','Income^3'};

% DATA.X = log(DATA.INC);
% INPUT.NamesX = {'log(Income)'};

% DATA.INC = log(DATA.INC);
% DATA.X = [DATA.INC DATA.INC.^2];
% INPUT.NamesX = {'log(Income)','log(Income)^2'};

% DATA.INC = log(DATA.INC);
% DATA.X = [DATA.INC DATA.INC.^2 DATA.INC.^3];
% INPUT.NamesX = {'log(Income)','log(Income)^2','log(Income)^3'};

DATA.X = exp(DATA.INC);
INPUT.NamesX = {'exp(Income)'};

% DATA.X = [DATA.GENdist];
% INPUT.NamesX = {'Distance'};

% DATA.X = [DATA.GENdist DATA.GENdist.^2];
% INPUT.NamesX = {'Distance','Distance^2'};

% DATA.X = [DATA.GENdist DATA.GENdist.^2 DATA.GENdist.^3];
% INPUT.NamesX = {'Distance','Distance^2','Distance^3'};

% DATA.X = log(DATA.GENdist);
% INPUT.NamesX = {'log(Distance)'};

% DATA.X = [exp(DATA.GENdist)];
% INPUT.NamesX = {'exp(Distance)'};

% DATA.X = [DATA.INC, log(DATA.GENdist)];
% INPUT.NamesX = {'Income','log(Distance)'};

% DATA.X = [DATA.INC, DATA.GENdist];
% INPUT.NamesX = {'Income','Distance'};

% normalization - help convergence:
DATA.X = DATA.X ./ nanstd(DATA.X);
DATA.X = DATA.X - nanmean(DATA.X);

% INPUT.SimStats = 1; % simulate descriptive statistics of the distribution

Countries = {'DK','EE','FI','DE','LV','LIT','PL','RU','SE'};

Results_h = [];
Results_f = [];

% fit normal distributions

% baseline:
% B0 = [-22140.4055087619,-23062.3441002009,-20151.8210457782,-21350.3182702583,-22377.0128892456,-22625.0934283892,-20000.2021154362,-21308.6092958683,-20421.9237351098;-22126.7754833352,-23077.0959410955,-20129.7890585624,-21337.5764146470,-22346.5446387134,-22611.5357037848,-20054.4258294164,-21363.8272693523,-20506.0854006131;-7.58266402296760,-6.89251784581461,-8.54278714371440,-7.98732638446000,-6.25370419304767,-6.40580261258610,-6.90621376934799,-0.0393820725158235,-5.21911832373860;78.5865701464730,80.1336306930860,78.5296134798315,83.9169226992397,83.7078606373169,81.3872086818072,86.2261627178165,80.1238474851452,58.4271851313024;-8.65405601080362,-6.41791302339614,-9.49776730970826,-8.60331711033221,-5.77784235226223,-6.17048396465429,-7.61763446819267,0.733979015691158,-6.41511355413086;99.7875239354677,99.5178326217532,94.7821337119112,99.7813605502369,99.4406283110777,99.8246977321565,106.465266977691,98.6553487937810,74.8315135126662];

% inc:
% B0 = [-21071.1594753465,-21803.3164924893,-19107.0143976638,-20189.8703589387,-21545.6167346875,-21814.8807345100,-19082.7185566519,-20344.0608400870,-19906.1588453244;-21157.4561215288,-22066.7293027798,-19228.4339708146,-20329.7703342757,-22345.8460037050,-21922.5840617865,-19258.6012124195,-20543.6007439974,-19933.0061881725;1.21114590545532,1.23608964019036,0.208645890156061,1.96869086491965,0.570141644030792,-0.312303815263957,0.679964856414411,3.84291615379606,-1.01966837229120;71.5190230540948,69.2155700376627,71.3361655608494,74.2085169382317,72.9795237025577,72.2210371907462,73.4245885454927,70.3610538890004,53.7088600284367;9.22635352290476,8.19153186368499,7.31133129255431,9.73795583779464,8.38088550622838,10.1110784416833,8.07029490391020,8.59495741816289,8.16438961750036;32.9048979077451,35.4504197802048,34.1981197719835,36.2150014852460,30.2379901755269,29.3329357874948,33.7324785207221,32.0323928857026,14.5933204787518;2.46209971457397,1.07931847258283,0.463332941746790,2.83323388717090,-5.11407598147833,-0.427111880690592,1.06902946297931,4.54831424870152,-0.530188771900397;89.6761914094234,86.9196449000316,87.9839741133848,89.9807995087525,100.963846627022,89.8614073229219,91.1612309054245,88.4720832715680,66.3066976165389;12.9464581165846,12.0357482835623,9.87495126838862,13.3181785405851,0.000530054220033738,13.4026319610913,10.9301294888115,11.3920048885500,9.45304479449181;39.4019811718304,39.5934244384395,39.7788165794196,40.5320961777436,0.000371590304905506,33.3073280871913,39.2273157883181,36.6103198330481,19.9952479394672];
% B0 = [-21071.1594753465,-21803.3164924893,-19107.0143976638,-20189.8703589383,-21545.6167346875,-21814.8807345100,-19082.7185566520,-20344.0608400869,-19906.1588453244;-21157.4561215288,-22066.7293027798,-19228.4339708146,-20329.7703342757,-21640.4422961217,-21922.5840617865,-19258.6012124195,-20543.6007439974,-19933.0061881725;1.21114590545532,1.23608964019349,0.208645890156061,1.96869086492949,0.570141644030792,-0.312303815263957,0.679964856414411,3.84291615379570,-1.01966837229120;71.5190230540948,69.2155700376635,71.3361655608494,74.2085169382344,72.9795237025577,72.2210371907462,73.4245885454927,70.3610538890003,53.7088600284367;9.22635352290476,8.19153186368734,7.31133129255431,9.73795583778900,8.38088550622838,10.1110784416833,8.07029490391020,8.59495741816220,8.16438961750036;32.9048979077451,35.4504197802058,34.1981197719835,36.2150014852477,30.2379901755269,29.3329357874948,33.7324785207221,32.0323928857018,14.5933204787518;2.46209971457392,1.07931847258283,0.463332941746790,2.83323388717090,1.09976337651316,-0.427111880691396,1.06902946297931,4.54831424870152,-0.530188771900397;89.6761914094234,86.9196449000316,87.9839741133848,89.9807995087525,90.4122557514216,89.8614073229217,91.1612309054245,88.4720832715680,66.3066976165389;12.9464581165842,12.0357482835623,9.87495126838862,13.3181785405851,11.3974910173574,13.4026319610897,10.9301294888115,11.3920048885500,9.45304479449181;39.4019811718305,39.5934244384395,39.7788165794196,40.5320961777436,34.4952172144696,33.3073280871916,39.2273157883181,36.6103198330481,19.9952479394672];

% inc inc^2:
% B0 = [-21071.1155073204,-21802.4025232905,-19091.2174297531,-20174.3671419720,-21545.6167346767,-21811.5715742478,-19080.7917886353,-20339.7857627243,-19905.5677028947;-21157.4558819602,-22056.6450954060,-19227.2311259381,-20326.0632923338,-21633.9946133960,-21918.1987679518,-19250.3088126085,-20533.5366297228,-19932.4791192134;1.21882860687367,1.02274651163006,0.452232886948266,3.04284743555872,0.570141644009172,-0.679907460953791,0.491394406230725,3.53035286875092,-0.650879458681602;71.5374828684477,68.9346944175091,73.3447429388806,75.2075136810493,72.9795237113024,72.2111284732522,72.8572165882717,69.9797725363348,53.4250246312188;9.14339165861351,10.1230633103118,10.4612806003330,9.40752065861092,8.38088551066107,16.9130217387484,9.43939347459134,15.7773137908122,9.91443963160877;0.130842799642128,-3.01621473085206,-3.72048667591155,2.46316270195975,8.99965312962657e-13,-8.98834804749699,-2.27948898207616,-10.2946217619556,-1.97642272289979;32.8347993772003,37.3373281787349,19.2958758008876,19.5348153690236,30.2379901851227,30.4198172852590,38.8578358529367,31.9829045509225,15.8174625571286;0.114919069046052,-3.21976734147983,24.2581042948742,25.3004906585639,9.51433842288468e-13,-2.11800100644691,-8.22261317563899,-0.411124835202148,-2.43359843642467;2.45630519390662,0.726305530091252,0.283009368149393,3.35557302885898,1.13882930979247,-0.532320223831617,0.710929800146565,4.61830349926960,-0.530674340497231;89.6898284778752,85.8056813608918,88.7572272500579,91.2073766078724,89.6003209339171,89.2230311695439,90.3147620016049,87.0433824187266,66.3086932165292;12.8653935021982,14.2363492779017,12.1237502790811,12.4919707393581,12.9543537113924,20.7519290974191,11.7148590003129,18.0774831128303,10.7392985620083;0.0873147862802057,-4.28753736857759,-3.10012002736207,2.35012385446495,-2.62653670817787,-9.65321306203421,-2.05617968294486,-9.75683279085835,-1.74150987798831;39.3349518084511,54.0150441002585,32.6674941623369,29.2788725902765,47.4573779284771,39.3549866053124,53.7409435775598,49.2304701103798,21.7795719465650;0.105422854550138,-21.3034324679059,10.9956542616700,17.2826520465671,-18.2377142936208,-8.91655721645814,-21.7092530343909,-18.0851286523566,-2.24190608238352];

% inc inc^2 inc^3:
% B0 = [-21003.4555865293,-21727.0413309731,-19004.3683798426,-20119.0328344201,-21480.5231090867,-21797.0796898235,-19018.8902350211,-20281.2917242387,-19905.1431166392;-21136.5525885366,-22024.3871477838,-19176.7923268917,-20297.6725772884,-21606.2643683719,-21909.4698740116,-19230.4387991735,-20523.7209507632,-19932.4788667241;0.231927353409765,1.21542147067148,0.785763892790382,2.75951747277034,0.822230415569531,-0.186376235843479,0.530663788358239,2.70193813119520,-0.566831278595646;73.8258447913017,67.3301199558772,68.4727787727514,71.2688696963221,72.1675525456024,71.7400114439464,70.9585863653889,71.7365692646985,53.3329997085343;-5.69289722762795,9.98592769990900,5.70759534464986,0.0541478770643201,7.31439795437759,-7.34714689133404,11.7014006883995,13.7720277076504,10.2456597555904;45.2824418160045,2.17395068719539,12.6515766959156,35.7758671247263,9.70998590275716,63.6549561486333,-4.42726218601818,3.42052934246520,-2.44795157296649;-31.2718260637598,-5.18700658598268,-12.1033213260700,-26.5751344167840,-10.4131879522881,-53.1051297541222,0.00318905458735410,-12.4537011610526,-3.83401640340640e-08;8.04030127632995,8.37279515426575,-11.5092370326172,-6.06939493295843,17.9454331675978,29.0219243751265,8.47360619059607,7.63858197279856,14.9689027488534;89.8409587199966,91.7891523872733,142.194771004783,119.910364607532,59.6413754804849,0.0710033665117015,84.5550271537121,82.1602222394722,-1.15775334681173;-75.4060022965080,-77.0805428758607,-111.032725397434,-90.1882933922022,-58.4076006823355,0.0763634564934410,-72.4011672758984,-67.3595009228673,-3.66953103219701e-07;1.29567536481354,1.02087306696258,0.906908767573566,3.31723158854588,1.02588683340814,-0.167482838199594,0.553954169700491,4.38720707375009,-0.535052561214555;88.8609041262641,84.6696520106318,84.7383531955688,87.7488384392060,88.3441072681157,89.3742584562292,89.5400729708911,86.7717316371161,66.3047884909859;5.25306628119220,16.1387900547860,9.20120649750331,-0.0104912535096907,6.69332318871848,-5.66718790754126,15.9448396263597,16.5026520611549,10.7254501712330;21.3927320296028,1.44237207740316,12.2403802802807,46.4008982080971,16.9544718349229,70.4818932142991,-5.92406100828519,-8.08836433464318,-1.73571863722408;-17.8097457131715,-7.92201000359407,-12.6427557416841,-34.8854984467119,-14.4962279672342,-58.5067233740739,-2.07670251160229e-05,0.358962980485379,-2.77775803488097e-08;40.5415101237925,13.1161652530102,-11.7272680813179,0.393122375992721,29.2581934611136,32.5057830956030,8.53811302456145,46.7517659492868,21.7725276047503;30.8856483494042,87.4596949810523,155.890610159750,116.654507067597,35.7959709423572,0.0833559264897825,96.2509531570976,-0.576233247490199,-2.23446091096974;-42.0249307997532,-74.4004102880939,-121.669522553871,-88.1753883549974,-41.1709067558173,0.0510649973988068,-79.6287340734415,-19.7747865376571,-3.62796572657586e-07];

% log(inc)
% B0 = [-21391.6799693348,-22219.1779371547,-19439.9749666162,-20571.9495521520,-21766.5933968863,-21987.6287781847,-19318.5005666722,-20604.6238825119,-20001.1794319455;-21399.0629275045,-22316.5571262403,-19465.5405970930,-20602.9157286069,-21777.0580414916,-22019.7984145286,-19404.9251060899,-20700.7059227087,-20030.5652152660;-3.06954724799376,-2.50681307780442,-3.74110113275360,-0.206140135140747,-1.49699715910622,-2.10417901181984,-2.27733942609760,2.51770999183408,-2.13671508038210;71.6488491167195,70.4466405104595,72.4946256731461,73.1970313981314,73.4976214169607,72.8716814536807,74.3557512524300,71.4606904075709,53.4350050261892;2.57824194362983,2.87495487588538,2.26349733523130,3.97128225041760,2.52477701379437,5.40931427258314,2.55020983887191,4.86690076754310,3.71206556145850;16.8127930431821,17.4498530237727,17.2043166037921,17.5201727831176,17.7616092307975,19.4196775544289,18.1117557558429,16.7613041248791,9.96759507257385;-2.54754753035753,-4.31942264610595,-4.06289744271392,-2.03593568339337,-1.37835287638028,-1.85807271483258,-1.83445026869750,1.51744277445533,-2.45681883840467;88.5661020077176,90.8086503644470,87.1448073420383,90.8232055642077,91.4160889975742,88.9502739679620,91.0075851611073,90.3202840395453,67.2999144233388;3.47938429946345,3.20863442122901,2.34453677128450,3.68292795747192,3.49767969983360,6.30175240696988,3.41368816889268,3.75361572601083,3.50223810585793;21.5430094616430,22.6727671053030,21.3233086769749,22.3343344804817,21.7046758956162,24.4298504265487,22.1661538154693,22.3979853066451,13.3235928601033];

% log(inc) + log(inc)^2:
% B0 = ;

% log(inc) + log(inc)^2 + log(inc)^3:
% B0 = ;

% exp(inc):
B0 = [-21176.5988873401,-21979.5958132845,-19173.6772120570,-20248.3399644731,-21702.2162014334,-21963.1664043857,-19225.8342550424,-20488.6428624610,-20002.9636239920;-21264.8638130130,-22245.2712125743,-19315.9565656265,-20393.5214269371,-21780.5002660890,-22052.8266352679,-19405.0721969534,-20693.2838865534,-20021.1135954033;4.17006890019754,2.96777887178282,2.27121332441205,6.64374740324660,1.75152959697467,1.02347856843859,1.86782114068843,4.80712893082287,-1.31864813129105;82.3652075721984,79.6878325552403,83.6571346802917,84.3356584054528,82.0079456149446,82.0971001800543,82.1995354610329,79.8269400842205,54.4550139153203;19.9870012405784,17.1117103631621,16.2855543027267,25.1628649483547,14.2970062085098,16.1611847599231,15.4315925545468,14.9289984782412,10.4301332321646;75.4022177197000,75.5399892216476,77.5494138095488,78.4083242767469,64.5288418467771,64.3011973379183,69.5810488030238,68.2238761650492,21.7456911351769;5.42281055366658,0.727152885673560,2.84619905129374,6.53022161785287,1.68637298862915,1.10684371585359,2.23576840215392,5.67498763903902,-0.936527041191521;107.213444201872,97.2947758608420,100.236069762960,106.656046044332,98.7905014012206,98.4497990959142,100.633423247098,96.5004941686815,67.7934876300492;26.6055279236673,17.4401958890798,20.3956574625626,29.1110256210815,18.3543890973653,22.1193879774496,19.9356258036620,18.4515849432060,13.0829887018868;95.9785892401080,80.7697092278965,87.1072448759080,96.5583388967811,62.6006605262692,61.4232175932443,76.6484609982591,73.0732202972206,29.4067683385258];

% dist:
% B0 = [-22055.4817745581,-22966.5804743444,-20078.7110681992,-21270.3599802859,-22275.7116126884,-22537.4793071660,-19923.1706851453,-20927.2630433729,-20347.1276432663;-22042.2055275618,-22986.9026473887,-20057.5629482191,-21257.1226303484,-22244.6787088844,-22521.3186116562,-19975.1804228750,-21089.5402876762,-20429.5992947538;-7.99959787954583,-7.75878474955865,-9.06446221618433,-9.21280291779259,-6.25868393079676,-6.38949749897022,-9.25282925149907,-0.515920669384829,-5.30965195097840;76.4418015424132,79.4237307871800,77.3049405004874,80.5490905462985,82.3153813956041,82.9138400500789,82.7717950945990,60.3281397817369,55.0262135930570;-11.6611829282127,-13.3504647690912,-10.9326399565469,-15.6384423229122,-15.8135905460870,-18.2169074680857,-14.5945602141253,1.67620877823494,-8.46723818449473;-2.35802392169221,-2.20452661638619,-2.60331997888459,-1.36506252199075,-1.86909284872634,0.000178019129461813,-1.75763686152349,-74.7338584285015,-1.24169044250735;-8.84908263151189,-10.1347808340470,-10.0530178979057,-11.6663431924763,-6.14366540459533,-7.67478748992556,-9.67146511715900,-0.0455025252556763,-6.39384932775013;95.3548741567719,99.3635028610460,93.2581552159254,99.5328684254064,99.3805807116191,99.5883839243918,99.3376481347841,76.2756208076943,69.2594731534998;-15.2465583955745,-22.7689189228595,-13.2434330359164,-23.3482324341830,-19.1489603957441,-17.0560764332733,-21.6983063198020,-1.13178655028663,-10.1646684490815;-2.81075661247021,4.95817456404304e-05,-3.10356821647958,5.16064775157407e-05,-2.28499272132056,-2.48679402274164,0.00772829403258669,-77.4390109372267,-1.72459724949460];

% dist dist^2:
% B0 = [-21702.5869544556,-22842.8496725159,-19743.2204544661,-20906.4370930937,-22153.1376213311,-22425.6113599810,-19842.7207408978,-20927.0037339397,-20289.9836725882;-21779.4343121885,-22737.5634136833,-19837.4038186434,-20893.3936425477,-21957.9232270808,-22457.3969260725,-19890.1258304530,-21089.0610902341,-20360.3418295181;-25.4489304652544,-6.45163010761002,-28.0113579421443,-32.6495796343230,-3.45909571146325,-4.67907180572415,-5.65330529140293,-0.248486723996264,-4.41017723370122;102.649207873817,77.3580963450708,104.263156015489,118.145277042880,77.5937119410737,77.8491259061650,77.9614528391859,59.7358329807725,54.1922418439946;29.1009003293687,0.0844863459649125,32.8656529562374,29.0107042386497,-7.64175612655246,-7.09833942119251,-7.08204714540820,0.625386350044521,-8.55868184757315;-249.600872269328,-0.00447956951723999,-272.282156161616,-323.923918338655,5.71023243130050,5.39243820810511,5.43910283009786,0.285020951153969,6.49032661104499;-113.181528948385,-20.8380443020467,-115.641009864373,-151.681814061017,-17.7567648664520,-17.6634177021734,-17.4481004781445,-73.2700802613217,-9.65941948062244;477.363144306415,11.3433189452049,500.965409417192,718.950587531669,8.95285645264765,8.84685965887365,8.67701320724185,2.84707035476558,4.23663767856063;-22.1465087869590,-5.83347480290062,-22.9594340407751,-42.5495691648332,-17.9210568273063,-7.17245519185203,-6.17145527829576,-0.123821119079180,-5.38471204539059;114.918763106335,104.569067827867,110.743334539475,147.459911628701,121.190337333521,98.1753816441468,95.7933035277633,76.7777574173570,68.9135682561581;22.1721188879494,-17.4056667983472,24.5751186002128,36.2994539169528,13.3097443175518,-29.0316856876869,-8.19499560550431,-1.39340746899122,-6.07938045652803;-206.540593279322,2.80458270007788,-205.094993612852,-430.532967193629,-179.891674321469,22.0996987401131,6.23029594004793,0.000627086932374542,0.366309380095079;-109.487218966882,-83.4922723593886,-102.250288089279,-185.405502484393,-117.272093193998,-7.60565607706616,-21.9467053552894,-79.2362534597419,-23.1167124192444;405.072535853232,246.200789822508,374.524226402401,918.244035423023,425.131031588614,-0.000145238496754279,11.0707122741728,-0.00258697918701140,26.0017748713107];

% dist dist^2 dist^3:
% B0 = [-21694.6217978372,-22545.6118291894,-19733.7871307732,-20902.6921373458,-21871.8086191088,-22244.8686685264,-19734.4366186222,-20869.6866691355,-20270.6677584161;-21771.9599396413,-22690.2193266594,-19802.7453250989,-20884.9348563879,-21947.1782930492,-22344.9463335231,-19851.9417598992,-21047.2390076455,-20338.4442287324;-34.1396094497028,-44.0981723638881,-33.0778635072363,-56.1021848391216,-29.2811223482306,-34.5545097354064,-31.2737430582480,93.2591417236184,-4.71040104528300;87.1951176006188,99.9578169928922,93.1109349512627,130.082441017051,94.4384894499908,91.4607925048800,94.7175876827416,75.6504881769186,53.5651173015768;4.59839725121912,16.6782640967846,20.3395595748523,31.8503942742857,14.2898551735237,5.61991044012472,18.7184795423978,-80.1592902430555,-12.0570729719957;-7.02270544433875,-108.828426028681,-133.984927874777,-305.711437314386,-128.182635725743,-27.1734054132720,-131.069160989764,1027.31434935376,15.5253871241926;-427.965877231634,-440.112167234021,-242.318508988315,-346.610514747661,-235.445858456648,-423.972651780739,-236.514783505072,-15.2365647297902,-6.21030553006560;-106.466761996588,-124.079420314370,-108.991463919266,-170.411329927396,-110.374602577911,-108.966123446237,-105.113762820836,-85.6651627650622,-14.9741508320299;442.904296526637,552.601000172675,456.702074659695,900.221697438784,445.502634115895,443.101677493959,432.674866678975,131.914079604678,23.9166025312568;-167.744316448937,-158.411789433371,-100.352347995279,-61.8144003828479,-109.225003373634,-162.857910246202,-105.540181324115,45.2692232337188,-15.0864586287542;-33.2630254627927,-44.1956064543610,-11.2702248335814,-56.0953415640329,-28.7174009774026,-6.29581969075214,-32.0655481207988,93.9686440876916,-4.48677580251605;104.117158195235,117.354476440322,91.7616515627204,141.179133962744,116.277292896160,99.5853070905254,110.835927253487,84.6518030183921,67.0003448178564;-0.450533208601409,20.4761243500485,28.5030352881368,28.5500703559549,7.22245792854113,-10.0719134239925,16.9778049418677,-80.4528638649392,-13.4972642886938;0.291551502684951,-182.350239682065,-266.890371686361,-305.865130363723,-98.1755422040806,4.32536291509945,-135.292674251988,1027.26592603824,31.1780036695240;-417.192470547496,-337.803084956409,236.710372772195,-346.701606606144,-228.053696237526,-33.5739195373837,-238.004386069063,-15.5371877600690,-20.2870958669425;-110.818938978330,-133.662861062356,-114.913270592985,-184.037147930583,-119.053426539312,-56.1083274737373,-107.491979048249,-78.9467161467153,-23.5743930345800;436.996549226647,627.793655711420,511.578940668321,898.208436270325,450.840070786265,99.4261860829857,425.067975449006,24.2922089659298,27.9869315534849;-171.067574946772,-240.324131890532,-415.026195177383,-61.1735099234532,-96.8099365986971,19.3110331866020,-115.730211203169,82.4300229674640,-13.5422650650481];
% ok, tylko se NaN
% B0 = [-21694.6217978372,-22545.6118291894,-19733.7871307732,-20902.6921373458,-21871.8086191088,-22244.8686685264,-19734.4366186222,-20869.6866691355,-20270.6677584161;-21771.9599396413,-22690.2193266594,-19802.7453250989,-20884.9348563879,-21947.1782930492,-22344.9463335231,-19851.9417598992,-21047.2390076455,-20338.4442287324;-34.1396094497028,-44.0981723638881,-33.0778635072363,-56.1021848391216,-29.2811223482306,-34.5545097354064,-31.2737430582480,93.2591417236184,-4.71040104528300;87.1951176006188,99.9578169928922,93.1109349512627,130.082441017051,94.4384894499908,91.4607925048800,94.7175876827416,75.6504881769186,53.5651173015768;4.59839725121912,16.6782640967846,20.3395595748523,31.8503942742857,14.2898551735237,5.61991044012472,18.7184795423978,-80.1592902430555,-12.0570729719957;-7.02270544433875,-108.828426028681,-133.984927874777,-305.711437314386,-128.182635725743,-27.1734054132720,-131.069160989764,1027.31434935376,15.5253871241926;-427.965877231634,-440.112167234021,-242.318508988315,-346.610514747661,-235.445858456648,-423.972651780739,-236.514783505072,-15.2365647297902,-6.21030553006560;-106.466761996588,-124.079420314370,-108.991463919266,-170.411329927396,-110.374602577911,-108.966123446237,-105.113762820836,-85.6651627650622,-14.9741508320299;442.904296526637,552.601000172675,456.702074659695,900.221697438784,445.502634115895,443.101677493959,432.674866678975,131.914079604678,23.9166025312568;-167.744316448937,-158.411789433371,-100.352347995279,-61.8144003828479,-109.225003373634,-162.857910246202,-105.540181324115,45.2692232337188,-15.0864586287542;-33.2630254627927,-44.1956064543610,-22.9594340407751,-56.0953415640329,-28.7174009774026,-6.29581969075214,-32.0655481207988,93.9686440876916,-4.48677580251605;104.117158195235,117.354476440322,110.743334539475,141.179133962744,116.277292896160,99.5853070905254,110.835927253487,84.6518030183921,67.0003448178564;-0.450533208601409,20.4761243500485,24.5751186002128,28.5500703559549,7.22245792854113,-10.0719134239925,16.9778049418677,-80.4528638649392,-13.4972642886938;0.291551502684951,-182.350239682065,-205.094993612852,-305.865130363723,-98.1755422040806,4.32536291509945,-135.292674251988,1027.26592603824,31.1780036695240;-417.192470547496,-337.803084956409,0,-346.701606606144,-228.053696237526,-33.5739195373837,-238.004386069063,-15.5371877600690,-20.2870958669425;-110.818938978330,-133.662861062356,-102.250288089279,-184.037147930583,-119.053426539312,-56.1083274737373,-107.491979048249,-78.9467161467153,-23.5743930345800;436.996549226647,627.793655711420,374.524226402401,898.208436270325,450.840070786265,99.4261860829857,425.067975449006,24.2922089659298,27.9869315534849;-171.067574946772,-240.324131890532,0,-61.1735099234532,-96.8099365986971,19.3110331866020,-115.730211203169,82.4300229674640,-13.5422650650481];
% B0 = [-21694.6217978372,-22545.6118291894,-19733.7871307732,-20902.6921373458,-21871.8086191088,-22244.8686685264,-19734.4366186222,-20869.6866691355,-20270.6677584161;-21771.9599396413,-22690.2193266594,-19837.4038186434,-20884.9348563879,-21947.1782930492,-22344.9463335231,-19851.9417598992,-21047.2390076455,-20338.4442287324;-34.1396094497028,-44.0981723638881,-33.0778635072363,-56.1021848391216,-29.2811223482306,-34.5545097354064,-31.2737430582480,93.2591417236184,-4.71040104528300;87.1951176006188,99.9578169928922,93.1109349512627,130.082441017051,94.4384894499908,91.4607925048800,94.7175876827416,75.6504881769186,53.5651173015768;4.59839725121912,16.6782640967846,20.3395595748523,31.8503942742857,14.2898551735237,5.61991044012472,18.7184795423978,-80.1592902430555,-12.0570729719957;-7.02270544433875,-108.828426028681,-133.984927874777,-305.711437314386,-128.182635725743,-27.1734054132720,-131.069160989764,1027.31434935376,15.5253871241926;-427.965877231634,-440.112167234021,-242.318508988315,-346.610514747661,-235.445858456648,-423.972651780739,-236.514783505072,-15.2365647297902,-6.21030553006560;-106.466761996588,-124.079420314370,-108.991463919266,-170.411329927396,-110.374602577911,-108.966123446237,-105.113762820836,-85.6651627650622,-14.9741508320299;442.904296526637,552.601000172675,456.702074659695,900.221697438784,445.502634115895,443.101677493959,432.674866678975,131.914079604678,23.9166025312568;-167.744316448937,-158.411789433371,-100.352347995279,-61.8144003828479,-109.225003373634,-162.857910246202,-105.540181324115,45.2692232337188,-15.0864586287542;-33.2630254627927,-44.1956064543610,-22.9594340407751,-56.0953415640329,-28.7174009774026,-6.29581969075214,-32.0655481207988,93.9686440876916,-4.48677580251605;104.117158195235,117.354476440322,110.743334539475,141.179133962744,116.277292896160,99.5853070905254,110.835927253487,84.6518030183921,67.0003448178564;-0.450533208601409,20.4761243500485,24.5751186002128,28.5500703559549,7.22245792854113,-10.0719134239925,16.9778049418677,-80.4528638649392,-13.4972642886938;0.291551502684951,-182.350239682065,-205.094993612852,-305.865130363723,-98.1755422040806,4.32536291509945,-135.292674251988,1027.26592603824,31.1780036695240;-417.192470547496,-337.803084956409,0,-346.701606606144,-228.053696237526,-33.5739195373837,-238.004386069063,-15.5371877600690,-20.2870958669425;-110.818938978330,-133.662861062356,-102.250288089279,-184.037147930583,-119.053426539312,-56.1083274737373,-107.491979048249,-78.9467161467153,-23.5743930345800;436.996549226647,627.793655711420,374.524226402401,898.208436270325,450.840070786265,99.4261860829857,425.067975449006,24.2922089659298,27.9869315534849;-171.067574946772,-240.324131890532,0,-61.1735099234532,-96.8099365986971,19.3110331866020,-115.730211203169,82.4300229674640,-13.5422650650481];

% B0 = [-21669.8572520686,-22545.1575343950,-19709.3174097296,-20884.8048080298,-21835.9506665358,-22219.7638116083,-19714.4292707397,-20714.3852141941,-20270.3257731605;-21769.5201114192,-22708.2832819031,-19837.6197569885,-20853.4605378422,-21979.9068187699,-22312.8147283304,-19821.2554000487,-20819.2466258331,-20337.8180659734;-7.47009556056024,-42.7806171013913,-10.2989104288521,-46.1100286609295,-7.29508521900229,-10.2524989920136,-8.99696807978437,10.9992883691668,-4.67585203826399;75.1889967693270,97.6362690898959,76.9058372485743,95.9115766207997,81.6384923545908,82.0472374406380,78.7470815570893,248.407781703706,53.5728688120845;14.5711229738093,17.1084413628642,26.9319508262907,17.9955936838384,20.7022974773926,32.3082345324004,25.6224880177707,-24.1982851705928,-11.9472173747154;-165.619938508344,-105.864943373559,-249.528309160208,-166.636280050740,-245.254041223365,-339.235631695393,-248.397701643817,355.068944692088,15.6192939722862;148.695004638688,-425.705262907112,220.905183618363,-387.679257799323,220.895162622445,303.443507713228,220.712291295689,-312.994581216337,-6.30536966434514;-105.545526906118,-125.088655567715,-116.284447101175,-157.536805593022,-130.698906835115,-122.649516833868,-108.395029421312,-272.913230887756,-14.9884190902314;474.369837150194,562.261356626373,540.340866728647,805.886250125858,627.206009325448,598.046928434593,499.932821981141,2740.25785100628,23.8895537461512;-384.937204652220,-202.113354902408,-440.881016057001,-413.622632540697,-514.413324411427,-492.112517318898,-407.709382509728,-903.917097602800,-15.1155819476792;-33.1247399317335,-8.52951101310205,-22.9594340807785,-16.3618558709287,-29.2593890856883,-3.75081134021680,-7.73956755588158,192.276303040360,-4.94407837162585;105.713683761019,94.4080619924475,110.743334296670,98.6188638831684,110.650665439498,95.6865587849376,93.9153091885117,365.442429360951,67.8865430731468;-0.549782891069690,18.5207377181940,24.5751167692879,44.8532520942130,7.39356949914183,-17.9112666631796,15.1886257719288,-22.1456370068382,-13.6762989261996;0.265856301040299,-190.530685259631,-205.095010407815,-487.945812166191,-103.551272159033,33.2069320055158,-146.140835570899,149.818404536100,31.2769848321147;-404.082283218261,170.182435691298,-1.47792545147423e-10,437.137551947997,-250.628428476745,-19.0355455150562,130.021931246490,2438.22409824431,-20.2157989185996;-109.678076250257,-114.950254836878,-102.250288902037,-202.113106358677,-126.214800681848,-55.5928182647694,-90.9016738200591,-309.438236893207,-23.5735172819549;419.020023336684,462.796025798008,374.524230232136,1108.46367184111,517.663245391272,112.107574948571,318.975936740594,3124.05863325353,27.9709607910602;-149.155705612213,-368.146978941952,-2.03546889620331e-10,-927.385269860349,-239.616848623282,-73.2208380792469,-246.962426502550,8.43044751346790,-13.5888248675869];

% log(dist)
% B0 = [-21391.6799693348,-22219.1779371547,-19439.9749666162,-20571.9495521520,-21766.5933968863,-21987.6287781847,-19318.5005666722,-20604.6238825119,-20001.1794319455;-21399.0629275045,-22316.5571262403,-19465.5405970930,-20602.9157286069,-21777.0580414916,-22019.7984145286,-19404.9251060899,-20700.7059227087,-20030.5652152660;-3.06954724799376,-2.50681307780442,-3.74110113275360,-0.206140135140747,-1.49699715910622,-2.10417901181984,-2.27733942609760,2.51770999183408,-2.13671508038210;71.6488491167195,70.4466405104595,72.4946256731461,73.1970313981314,73.4976214169607,72.8716814536807,74.3557512524300,71.4606904075709,53.4350050261892;2.57824194362983,2.87495487588538,2.26349733523130,3.97128225041760,2.52477701379437,5.40931427258314,2.55020983887191,4.86690076754310,3.71206556145850;16.8127930431821,17.4498530237727,17.2043166037921,17.5201727831176,17.7616092307975,19.4196775544289,18.1117557558429,16.7613041248791,9.96759507257385;-2.54754753035753,-4.31942264610595,-4.06289744271392,-2.03593568339337,-1.37835287638028,-1.85807271483258,-1.83445026869750,1.51744277445533,-2.45681883840467;88.5661020077176,90.8086503644470,87.1448073420383,90.8232055642077,91.4160889975742,88.9502739679620,91.0075851611073,90.3202840395453,67.2999144233388;3.47938429946345,3.20863442122901,2.34453677128450,3.68292795747192,3.49767969983360,6.30175240696988,3.41368816889268,3.75361572601083,3.50223810585793;21.5430094616430,22.6727671053030,21.3233086769749,22.3343344804817,21.7046758956162,24.4298504265487,22.1661538154693,22.3979853066451,13.3235928601033];

% exp(dist) % truncated to 0.975
% B0 = [-22100.4209275197,-23019.2499176339,-20117.5232026573,-21312.1148252983,-22333.8872197482,-22579.0312288431,-19957.1048804668,-21304.5068468616,-20376.8986656909;-22089.7073852287,-23033.1703164096,-20095.9127788324,-21299.8666718939,-22303.5162644809,-22568.8527384365,-20007.6091565313,-21360.7514464207,-20458.2941887936;-7.27288989319119,-6.87930393702697,-8.24296494142741,-7.05909139282596,-4.68369561288022,-5.74594504968220,-6.39453885481912,-2.31447299653804,-4.43877677296512;76.9879714310044,79.9628131678863,77.8627986755961,80.8822279943382,82.8028179805941,82.9290009422170,83.0066687090001,79.1505507331128,54.4201172493569;0.313986502653146,0.277679589881441,0.410824719066610,0.296041909165197,0.00908819285541866,0.145487474334673,0.216341057049432,-0.565067728864897,0.0787419354518519;-6.63666212774756,-6.90528289747531,-6.72102205213036,-6.98993295417448,-7.12011068548489,-7.15562552365900,-7.16962261180337,0.133129894845106,-4.60784533731774;-7.88179935626389,-8.23455702224627,-9.05747713854704,-8.28285295908495,-4.78500580786007,-5.78374545737142,-6.50523989420491,-2.36120237329520,-5.65297621622629;95.9847579825183,98.8834108991452,93.9314571960243,98.8923513392594,100.055414499471,98.9813561597262,98.6742560163138,99.0286057346993,69.4730667401193;0.291791902223798,0.286832927023372,0.407028971175098,0.229039074702530,-0.0234067681362861,0.0867177054460668,0.157140729840145,-7.59901430499160e-15,0.113733890302349;-8.28333198298315,-8.52396915551536,-8.10256876100814,-8.48600205163735,-8.62881754671080,-8.54474657472455,-8.52031528927768,1.94969048959542e-14,-5.89930691120863];

% inc log(dist):
% B0 = [-2086.80357724788,-1121.08433579006,-3968.31177736451,-2754.98219506857,-1351.20165316770,-1216.30067992555,-3920.18686419861,-2297.02874935500,-2620.59548446229;-2121.13990959937,-1113.79763657677,-4017.39667200117,-2837.73483782182,-1380.48577951651,-1234.93578536808,-3839.87740609161,-2193.83673406002,-2671.20481612940;-2.37153646170232,2.86015735978220,2.50208400641504,1.84221043762367,3.94912663775486,15.3041248210878,4.25401205914861,-30.1664448336998,35.1584561383564;69.1271809630982,57.5019265526369,71.8537232325631,59.3219309724503,11.7938628258104,20.6938753762501,31.6831045453704,61.7296889212434,94.4961330107803;2.45236683925994,7.67776183337045,8.22302185633922,1.85995458849483,7.05642471822749,17.0912749541063,8.30372130259509,-3.85434284312610,16.9969845738117;-6.86372917237202,0.580076165105950,-10.1116663311431,-0.000172008836506932,-2.89346863005392,1.62372130852745,-0.762438088733431,-6.52951364656489,-6.84241140782534;20.5484483958201,-2.58261466569706,9.52230000209118,2.11281738390640,-1.16919491878619,-6.58924606181389,0.104024965847434,22.9732854039005,54.3919360191395;3.59074293349503,-1.47394052144652,6.97276914946823,-14.2755711966637,-3.45190343131811,-10.4497203084423,-3.57381773243444,2.37215561463140,-15.7524708170154;-6.01218726226790,4.28505071053135,0.0441416723867003,1.27661648653308,4.30119637981611,24.2871542441977,6.78646004479928,-28.5035152638815,42.9574964750532;80.5253901807057,75.1084279472639,103.546807455168,89.3396019770257,18.1560699235394,23.2313406143395,38.3164399579008,64.2979149734216,114.836670003761;3.95094999132212,5.84071114567556,12.4181368703250,1.16552143546089,8.70640192244557,25.3523657824363,11.2852156604956,-0.400590475523352,19.4050712196660;-8.93916455506725,0.168636828830945,-13.2625279051902,1.27357174376391,-4.54148961209229,0.553460428997844,-1.15899823252081,-4.90411890888329,-12.2562014989733;18.0137789666424,12.6377957369317,5.64875756393936,6.98478829520520,1.43627623425585,-10.9108398704574,2.22797471806400,20.9497807569090,31.9118573812719;6.50027613306610,-4.87555196099131,2.67677295110426,-25.0099354189048,-0.389318482044049,-12.0453097125590,-3.96569434283371,-0.881330096798522,-53.8080004643786];

% inc dist:
% B0 = [-21017.4441403595,-21752.2971111833,-19060.5212316758,-20139.6216273629,-21483.8915145784,-21759.3810702739,-19037.1070099346,-20240.9669375972,-19859.3501492718;-21100.2478208544,-22013.1092058183,-19179.9299847230,-20276.4838732101,-21572.0805527422,-21864.0572593948,-19211.3025464791,-20416.8155109578,-19884.8510365409;0.643937660843498,0.848929997268234,-0.347069712096819,0.806421518325306,-0.542248975419312,-0.430951313283337,-1.45460530831404,3.79865812613764,-0.814765474701564;71.1068455571391,68.8973762051843,70.9399848930728,74.0907179013607,72.6242768741100,71.6144253665087,75.3707139504480,61.7006117060527,52.8998846762716;8.88219710543208,7.84567914751769,7.09293484238382,9.07943430068524,7.60951701246165,10.0207510497144,6.73237250287284,8.29558859251353,7.98898414705847;-8.03321943408122,-7.15639595728701,-7.66761250451412,-8.60365634346088,-11.8262918056057,-8.57784955191388,-9.45559419311985,-1.09371947878289,-6.08176989702843;32.6552155665198,35.2337701591284,33.9812355243779,36.2790824681929,29.9825971367923,29.0076055573330,35.7196393953778,28.5997792429666,14.1666798224811;-0.122622969673046,0.148479430976005,-0.121721035044958,0.260230597363525,1.09238910856225,-0.163073994512468,0.688089545206558,-32.2043260627450,-0.610026573425471;0.822888079497208,0.673020569103198,0.0489003799474570,1.68627352868041,0.536201381826140,-1.11084996306534,-0.0948653945362206,5.20634570750086,-0.909063466379053;90.4130823478911,86.3084249587397,87.1981757736008,89.3551803287735,89.6254058747286,89.0735947172426,90.5667678559037,76.6011893570940,65.7980831065752;11.8953837506651,11.4797089851107,9.66114108291238,12.5618849726491,10.2179121012544,12.5166642981196,10.2600875817083,11.1619894603331,9.05930687648766;-9.80296603886684,-8.09736765642208,-8.71036547864939,-9.65545199400809,-10.9877982773293,-9.84843471241569,-9.83517322537661,0.273762028915776,-6.89331376468647;38.6827080822174,39.1533199167637,39.3600933448721,40.3236387777502,34.2211250925518,32.8744348117706,38.8811606000251,31.8403315089836,19.8055371679401;-0.771663045877989,-0.806434620881054,-0.771650197641424,-0.544846267548486,-0.960724084857633,-1.01743352761828,-0.567077052931213,-44.9949946782464,-0.993579400704143];

% 0 -> 1
% B0 = [B0(1:4,:);0*ones(2,9);B0(5:6,:);0*ones(2,9)];

% 1 -> 2
% B0 = [B0(1:5,:);0*ones(1,9);B0(6,:);0*ones(1,9);B0(7:9,:);0*ones(1,9);B0(10,:);0*ones(1,9)];

% 2 -> 3
% B0 = [B0(1:6,:);0*ones(1,9);B0(7:8,:);0*ones(1,9);B0(9:12,:);0*ones(1,9);B0(13:14,:);0*ones(1,9)];

% LL0_new = reshape(B02(1:2,:)',[1,18]);
% B0_h_new = B02(3:(size(B02,1)-2)./2+2,:);
% B0_f_new = B02((size(B02,1)-2)./2+3:end,:);
% idx = reshape(LL0 < LL0_new,[9,2])';
% B0_h(:,idx(1,:)) = B0_h_new(:,idx(1,:))
% B0_f(:,idx(2,:)) = B0_f_new(:,idx(2,:))
% B0 = [reshape(LL1,[9,2])'; B0_h; B0_f];

INPUT.Algorithm = 'sqp';% 'search' 'interior-point' 'sqp' 'active-set' 'trust-region-reflective'

LLdif = -1;
while min(LLdif(:)) < -1e-6
    
    LL0 = reshape(B0(1:2,:)',[1,18]);
    B0_h = B0(3:(size(B0,1)-2)./2+2,:);
    B0_f = B0((size(B0,1)-2)./2+3:end,:);
    
    Results_h = [];
    Results_f = [];
    for ctr = 1:9
        INPUT.bounds = DATA.bounds_h(DATA.CTR ~= ctr,:);
        INPUT.X = DATA.X(DATA.CTR ~= ctr,:);
        INPUT.WT = DATA.W(DATA.CTR ~= ctr,:);
        Results_tmp = DistFit(INPUT,0,B0_h(:,ctr));
        Results_tmp.CTR = Countries{ctr};
        Results_h = [Results_h,Results_tmp];
        
        INPUT.bounds = DATA.bounds_f(DATA.CTR ~= ctr,:);
        Results_tmp = DistFit(INPUT,0,B0_f(:,ctr));
        Results_tmp.CTR = Countries{ctr};
        Results_f = [Results_f,Results_tmp];
    end
    clear Results_tmp
    % Results_h = orderfields(Results_h,[17,1:16]);
    % Results_f = orderfields(Results_f,[17,1:16]);
    Results_h = orderfields(Results_h,[numel(fields(Results_h)),1:numel(fields(Results_h))-1]);
    Results_f = orderfields(Results_f,[numel(fields(Results_f)),1:numel(fields(Results_f))-1]);
    
    
    % B0_h = B0_h2;
    % B0_f = B0_f2;
    
    LL1 = [Results_h.fval,Results_f.fval];
    LLdif = reshape(LL0 - LL1,[9,2])'
    B0_h_new = [Results_h.beta];
    B0_h(:,LLdif(1,:)<0) = B0_h_new(:,LLdif(1,:)<0);
    B0_f_new = [Results_f.beta];
    B0_f(:,LLdif(2,:)<0) = B0_f_new(:,LLdif(2,:)<0);
    % LL0(LL1>LL0) = LL1(LL1>LL0);
    
    % LL0_new = [Results_h.fval,Results_f.fval];
    % idx = reshape(LL0 < LL0_new,[2,9]);
    % B0_h_new = [Results_h.beta];
    % B0_f_new = [Results_f.beta];
    % B0_h(:,idx(1,:)) = B0_h_new(:,idx(1,:))
    % B0_f(:,idx(2,:)) = B0_f_new(:,idx(2,:))
    %
    % B0_h = B0_h_new;
    % B0_f = B0_f_new;
    B0 = [reshape(LL1,[9,2])'; B0_h; B0_f];
    
    % save FT_distance
    
end

% LL0 = reshape(LL0, [9,2])

LL1 = reshape(LL1, [9,2])



%% simulate moments of spike distribution


samp1 = 1e4;
samp2 = 1e4; % 1e5 x 1e5 = ? min
WTP_stats = zeros(samp2,4);
% for ctr = 1:9
%     X_mean_tmp(ctr,:) = nanmean(DATA.X(DATA.CTR == ctr,:));
% end

% inc:
% X_mean_tmp = INCm';
% X_mean_tmp = INCm12;
% X_mean_tmp = INCm123;
% X_mean_tmp = logINCm';
X_mean_tmp = expINCm';
% X_mean_tmp = GENdistm';
% X_mean_tmp = GENdistm12;
% X_mean_tmp = GENdistm123;
% X_mean_tmp = logGENdistm';
% X_mean_tmp = expGENdistm';
% X_mean_tmp = [INCm', GENdistm'];




% X_mean_tmp(1:9) = 0;
% INPUT.X = zeros(size(DATA.bounds_f,1),0);

for ctr = 1:9
    % half:
    betas_ctr = mvnrnd(Results_h(ctr).beta,Results_h(ctr).ihess,samp1);
    mu_tmp = betas_ctr(:,1);
    sigma_tmp = betas_ctr(:,2);
    mu_exp_tmp = betas_ctr(:,3:2+size(INPUT.X,2));
    sigma_exp_tmp = betas_ctr(:,3+size(INPUT.X,2):end);
    parfor i = 1:samp2
        WTP_i = normrnd(mu_tmp(i) + X_mean_tmp(ctr,:)*mu_exp_tmp(i,:)',max(sigma_tmp(i) + X_mean_tmp(ctr,:)*sigma_exp_tmp(i,:)',eps),[samp2,1]);
        WTP_i(WTP_i < 0) = 0;
        WTP_stats(i,:) = [nanmean(WTP_i), nanmedian(WTP_i), nanstd(WTP_i), normcdf(0,mu_tmp(i),sigma_tmp(i))];
    end
    Results_h(ctr).WTP_stats = [nanmedian(WTP_stats);nanstd(WTP_stats)]; % [est,se] x [mean,median,s.d.,p(WTP=0)] x ctr(predicted)
    % full:
    betas_ctr = mvnrnd(Results_f(ctr).beta,Results_f(ctr).ihess,samp1);
    mu_tmp = betas_ctr(:,1);
    sigma_tmp = betas_ctr(:,2);
    mu_exp_tmp = betas_ctr(:,3:2+size(INPUT.X,2));
    sigma_exp_tmp = betas_ctr(:,3+size(INPUT.X,2):end);
    parfor i = 1:samp2
        WTP_i = normrnd(mu_tmp(i) + X_mean_tmp(ctr,:)*mu_exp_tmp(i,:)',max(sigma_tmp(i) + X_mean_tmp(ctr,:)*sigma_exp_tmp(i,:)',eps),[samp2,1]);
        WTP_i(WTP_i < 0) = 0;
        WTP_stats(i,:) = [nanmean(WTP_i), nanmedian(WTP_i), nanstd(WTP_i), normcdf(0,mu_tmp(i),sigma_tmp(i))];
    end
    Results_f(ctr).WTP_stats = [nanmedian(WTP_stats);nanstd(WTP_stats)];
end;
clear betas_ctr mu_tmp sigma_tmp WTP_i WTP_stats samp1 samp2 ctr i

WTP_stats_h = reshape([Results_h.WTP_stats],[2,4,9]); % [est,se] x [mean,median,s.d.,p(WTP=0)] x ctr(to)
WTP_stats_f = reshape([Results_f.WTP_stats],[2,4,9]);


%% calculate TE and MTL

% i - transfered (from)
% j - observed (to)

means_h = squeeze(WTP_stats_h(1,1,:));
means_f = squeeze(WTP_stats_f(1,1,:));
se_h = squeeze(WTP_stats_h(2,1,:));
se_f = squeeze(WTP_stats_f(2,1,:));

means_h_true = [34.9217347400289;23.7658786821308;36.5000127910189;20.8608845716342;5.09839616980444;8.41030888407497;12.2713811955221;8.86482401601798;80.4895466599075];
means_f_true = [37.3408325456081;31.2093810666659;48.0326520467845;30.3884353696430;6.29259147736952;11.2805172166923;15.2454776383295;10.1060648532687;97.6615490634260];
se_h_true = [1.65381080406161;1.66263268186932;1.18706325727988;0.854461839379708;0.315806129758782;0.543096699561541;0.445378289309211;0.519186221316755;3.76317483134154];
se_f_true = [1.76686647939876;2.16294330267589;1.51687672038295;1.23234199755165;0.391331531757082;0.733959580997720;0.547980339388188;0.580193992237844;3.76619603870486];


% BT - one-to-one
TEh1 = TEMatrix(means_h,means_h_true);
TEf1 = TEMatrix(means_f,means_f_true);
MTLh1 = MTLMatrix(means_h,se_h,means_h_true,se_h_true);
MTLf1 = MTLMatrix(means_f,se_f,means_f_true,se_f_true);

% TEf1(diag(ones(size(TEf1,1),1))==1) = NaN;
% TEh1(diag(ones(size(TEh1,1),1))==1) = NaN;
% 
% MTLf1(diag(ones(size(MTLf1,1),1))==1) = NaN;
% MTLh1(diag(ones(size(MTLh1,1),1))==1) = NaN;

AvgTE = [...
    nanmean(abs(reshape([TEf1,TEh1],[2*9,1])));...
    nanmedian(abs(reshape([TEf1,TEh1],[2*9,1])))]

AvgMTL = [...
    nanmean(abs(reshape([MTLf1,MTLh1],[2*9,1])));...
    nanmedian(abs(reshape([MTLf1,MTLh1],[2*9,1])))]

out = [LL1, zeros(9,1), TEf1, zeros(9,1), TEh1, zeros(9,1), MTLf1, zeros(9,1), MTLh1, zeros(9,1), [AvgTE;zeros(7,1)], zeros(9,1), [AvgMTL;zeros(7,1)]];


